--[[
-- Name: TinyTip
-- Author: Thrae of Maelstrom (aka "Matthew Carras")
-- Release Date: 6-25-06
--
-- This file handles the position and anchoring of the
-- tooltip. If you don't care about these options, then
-- this file can safely be deleted or removed from the TOC.
--]]

-- blatantly stolen from PerfectTooltip by cladhaire!
local x,y,screenWidth, screenHeight, AnchorCursor
local function SmartSetOwner(owner, setX, setY, tooltip, scale)
	if (not owner) then owner = getglobal('UIParent') end
	if (not tooltip) then tooltip = this end
	if (not setX) then setX = 0 end
	if (not setY) then setY = 0 end

	x,y = owner:GetCenter()
    local left = owner:GetLeft()
	local right = owner:GetRight()
    local top = owner:GetTop()
    local bottom = owner:GetBottom()
	screenWidth = UIParent:GetWidth()
    screenHeight = UIParent:GetHeight()
    if not scale then scale = owner:GetScale() end
    if (x~=nil and y~=nil and left~=nil and 
	right~=nil and top~=nil and bottom~=nil 
	and screenWidth>0 and screenHeight>0) then
        setX = setX * scale
        setY = setY * scale
        x = x * scale
		y = y * scale 
		left = left * scale 
		right = right * scale 
		width = right - left 
		top = top * scale 
		bottom = bottom * scale 
		local anchorPoint 
		
		if (y <= (screenHeight * (1/2))) then 
			top = top + setY 
			anchorPoint = "TOP" 
			if (top < 0) then 
				setY = setY - top 
			end 
		else 
			setY = -setY 
			bottom = bottom + setY 
			anchorPoint = "BOTTOM" 
			if (bottom > screenHeight) then 
				setY = setY + (screenHeight - bottom) 
			end 
		end 
		
		if (x <= (screenWidth * (1/2))) then 
			left = left + setX 
			if (anchorPoint == "BOTTOM") then 
				anchorPoint = anchorPoint.."RIGHT" 
				setX = setX - width 
				if (left < 0) then 
					setX = setX - left 
				end 
			else 
				anchorPoint = anchorPoint.."LEFT" 
				if (left < 0) then 
					setX = setX - left 
				end 
			end 
		else 
			setX = -setX 
			right = right + setX 
			if (anchorPoint == "BOTTOM") then 
				anchorPoint = anchorPoint.."LEFT" 
				setX = setX + width 
				if (right > screenWidth) then 
					setX = setX - (right - screenWidth) 
				end 
			else 
				anchorPoint = anchorPoint.."RIGHT" 
				if (right > screenWidth) then 
					setX = setX + (screenWidth - right) 
				end 
			end 
		end 
		
		scale = tooltip:GetScale() 
		if (scale) then 
			setX = setX / scale 
			setY = setY / scale 
		end
		
		tooltip:SetOwner(owner, "ANCHOR_"..anchorPoint, setX, setY)
	end
end

-- by me! Uses completely transparent tooltip to anchor.
local function SmartAnchorCursor(tooltip, owner, offx, offy, scale)
	if not owner then owner = getglobal("UIParent") end
	if not scale then scale = owner:GetScale() end
	if not TinyTipUtil_Tooltip:IsOwned(owner) then
		TinyTipUtil_Tooltip:ClearAllPoints()
		TinyTipUtil_Tooltip:SetOwner(owner, "ANCHOR_CURSOR")
		TinyTipUtil_Tooltip:SetBackdropColor(0,0,0,0)
		TinyTipUtil_Tooltip:SetBackdropBorderColor(0,0,0,0)
		TinyTipUtil_Tooltip:SetText("TT",0,0,0,0)
	end
	tooltip:SetOwner( owner, "ANCHOR_NONE")
	tooltip:SetPoint("BOTTOM", TinyTipUtil_Tooltip, "TOP", (offx or 0) * scale, (offy or 0) * scale)
end

local Original_GameTooltip_SetDefaultAnchor, SetDefaultAnchorIsHooked, scale
local function SetDefaultAnchor(tooltip,owner)
	Original_GameTooltip_SetDefaultAnchor(tooltip,owner)
	y = TinyTipUtil_GetDB("Anchor")
	scale = owner:GetScale()
	if y then
		if AnchorCursor then
			if owner ~= getglobal('UIParent') then
				if AnchorCursor == 2 or AnchorCursor == 4 then
					SmartSetOwner(owner, 0, 0, tooltip, scale)
				elseif tooltip == "GameTooltip" then
					tooltip:SetOwner(owner, "ANCHOR_NONE")
					tooltip:SetPoint("BOTTOMRIGHT", getglobal("UIParent"), "BOTTOMRIGHT",
							(TinyTipUtil_GetDB("OffX") or 0) * scale, 
							(TinyTipUtil_GetDB("OffY") or 0) * scale)
				end
			elseif AnchorCursor >= 3 and tooltip == "GameTooltip" then
				tooltip:ClearAllPoints()
				SmartAnchorCursor(tooltip, owner, 
							  TinyTipUtil_GetDB("OffX"), TinyTipUtil_GetDB("OffY"), scale)
			elseif tooltip == "GameTooltip" then
				tooltip:ClearAllPoints()
				tooltip:SetOwner(owner, "ANCHOR_CURSOR")
			end
		elseif tooltip == "GameTooltip" then
			tooltip:SetOwner(owner, "ANCHOR_NONE")
			tooltip:SetPoint(y, owner, y,
							(TinyTipUtil_GetDB("OffX") or 0) * scale, 
							(TinyTipUtil_GetDB("OffY") or 0) * scale)
		end
	end
end

function TinyTipAnchor_Hook()
	if not SetDefaultAnchorIsHooked then
		Original_GameTooltip_SetDefaultAnchor = GameTooltip_SetDefaultAnchor
		GameTooltip_SetDefaultAnchor = SetDefaultAnchor
		SetDefaultAnchorIsHooked = true
	end
	if string.find(TinyTipUtil_GetDB("Anchor") or "", "CURSOR", 1, true) then
		if TinyTipUtil_GetDB("Anchor") == "CURSOR" then AnchorCursor = 1
		elseif TinyTipUtil_GetDB("Anchor") == "SMARTCURSOR" then AnchorCursor = 2
		elseif TinyTipUtil_GetDB("Anchor") == "OFFSETCURSOR" then AnchorCursor = 3
		elseif TinyTipUtil_GetDB("Anchor") == "SMARTOFFSETCURSOR" then AnchorCursor = 4
		end
	end
end
