--[[
-- Name: TinyTip
-- Author: Thrae of Maelstrom (aka "Matthew Carras")
-- Release Date: 6-25-06
--
-- This file gives utility functions provided by the
-- rest of the addon. This file must be perserved.
--]]

local db, ddb
TinyTipUtil_Frame = nil -- global

-- Print out a message, probably to ChatFrame1
function TinyTipUtil_Msg(msg)
	DEFAULT_CHAT_FRAME:AddMessage( "|cFFFFCC00TinyTip:|r " .. msg )
end

--[[ Initialize the SavedVariables database.
	* If empty, copies the default table values to it.
	* If mismatched versions, prunes out invalid entries
	  and adds new ones.
	* Otherwise just sets the local db.
--]]
function TinyTipUtil_InitDB(defaults,allopt)
	local k,v
	if not TinyTipDB then
		TinyTipUtil_Msg( TinyTipLocale_InitDB1 )
		TinyTipDB = {}
		for k,v in defaults do
			TinyTipDB[k] = v
		end
		db = TinyTipDB
		TinyTipUtil_Msg( TinyTipLocale_InitDB2 )
	elseif TinyTipDB._v ~= defaults._v then
		TinyTipUtil_Msg( TinyTipLocale_InitDB3 )
		local k
		for k,_ in TinyTipDB do -- prune
			if not allopt[k] then
				TinyTipDB[k] = nil
			end
		end
		for k,v in allopt do -- add
			if not TinyTipDB[k] or type(TinyTipDB[k]) ~= type(v) then 
				TinyTipDB[k] = defaults[k]
			end
		end
		TinyTipDB["_v"] = defaults["_v"]
		TinyTipUtil_Msg( TinyTipLocale_InitDB4 )
		db = TinyTipDB
	else
		TinyTipUtil_Msg( TinyTipLocale_InitDB5 )
		db = TinyTipDB
	end
	ddb = defaults

	-- nil out what we don't need any longer
	TinyTipLocale_InitDB1 = nil
	TinyTipLocale_InitDB2 = nil
	TinyTipLocale_InitDB3 = nil

	-- then finally, nil out the function itself
	TinyTipUtil_InitDB = nil
end

-- Sets database back to default values.
function TinyTipUtil_DefaultDB()
	local k,v
	if db and db._v == ddb._v then
		for k,v in ddb do
			db[k] = v
		end
		TinyTipUtil_Msg( TinyTipLocale_DefaultDB1 )
	else
		TinyTipUtil_Msg( TinyTipLocale_DefaultDB2 )
	end
end

-- Both of these are obvious.
function TinyTipUtil_GetDB(k)
	return db[k]
end

function TinyTipUtil_SetDB(k,v)
	db[k] = v
end

function TinyTipUtil_ToggleDB(k)
	if db[k] then db[k] = nil
	else db[k] = true end
end

-- Return color format in HEX from Blizzard percentage RGB
-- for the class.
local tmp
function TinyTipUtil_ColorPlayer(unit)
	_,tmp=UnitClass(unit)
	if tmp and RAID_CLASS_COLORS[tmp] then
		return string.format("%2x%2x%2x", RAID_CLASS_COLORS[tmp].r*255, 
										  RAID_CLASS_COLORS[tmp].g*255, 
										  RAID_CLASS_COLORS[tmp].b*255)
	else
		return "FFFFFF"
	end
end
